# Copyright Amazon.com Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0
# Do not modify this file. This file is machine generated, and any changes to it will be overwritten.

from aws_cryptography_primitives.internaldafny.generated.AwsCryptographyPrimitivesTypes import (
    IAwsCryptographicPrimitivesClient,
)
from typing import Callable, TypeVar, cast

from .config import Config, CryptoConfig
from .dafny_protocol import DafnyRequest, DafnyResponse
from .plugin import set_config_impl
from smithy_python.exceptions import SmithyRetryException
from smithy_python.interfaces.interceptor import Interceptor, InterceptorContext
from smithy_python.interfaces.retries import RetryErrorInfo, RetryErrorType

from .config import Plugin
from .deserialize import (
    _deserialize_aes_decrypt,
    _deserialize_aes_encrypt,
    _deserialize_aes_kdf_counter_mode,
    _deserialize_compress_public_key,
    _deserialize_decompress_public_key,
    _deserialize_derive_shared_secret,
    _deserialize_digest,
    _deserialize_ecdsa_sign,
    _deserialize_ecdsa_verify,
    _deserialize_generate_ecc_key_pair,
    _deserialize_generate_ecdsa_signature_key,
    _deserialize_generate_random_bytes,
    _deserialize_generate_rsa_key_pair,
    _deserialize_get_public_key_from_private_key,
    _deserialize_get_rsa_key_modulus_length,
    _deserialize_h_mac,
    _deserialize_hkdf,
    _deserialize_hkdf_expand,
    _deserialize_hkdf_extract,
    _deserialize_kdf_counter_mode,
    _deserialize_parse_public_key,
    _deserialize_rsa_decrypt,
    _deserialize_rsa_encrypt,
    _deserialize_validate_public_key,
)
from .errors import ServiceError
from .models import (
    AESDecryptInput,
    AESEncryptInput,
    AESEncryptOutput,
    AesKdfCtrInput,
    CompressPublicKeyInput,
    CompressPublicKeyOutput,
    DecompressPublicKeyInput,
    DecompressPublicKeyOutput,
    DeriveSharedSecretInput,
    DeriveSharedSecretOutput,
    DigestInput,
    ECDSASignInput,
    ECDSAVerifyInput,
    GenerateECCKeyPairInput,
    GenerateECCKeyPairOutput,
    GenerateECDSASignatureKeyInput,
    GenerateECDSASignatureKeyOutput,
    GenerateRSAKeyPairInput,
    GenerateRSAKeyPairOutput,
    GenerateRandomBytesInput,
    GetPublicKeyFromPrivateKeyInput,
    GetPublicKeyFromPrivateKeyOutput,
    GetRSAKeyModulusLengthInput,
    GetRSAKeyModulusLengthOutput,
    HMacInput,
    HkdfExpandInput,
    HkdfExtractInput,
    HkdfInput,
    KdfCtrInput,
    ParsePublicKeyInput,
    ParsePublicKeyOutput,
    RSADecryptInput,
    RSAEncryptInput,
    ValidatePublicKeyInput,
    ValidatePublicKeyOutput,
)
from .serialize import (
    _serialize_aes_decrypt,
    _serialize_aes_encrypt,
    _serialize_aes_kdf_counter_mode,
    _serialize_compress_public_key,
    _serialize_decompress_public_key,
    _serialize_derive_shared_secret,
    _serialize_digest,
    _serialize_ecdsa_sign,
    _serialize_ecdsa_verify,
    _serialize_generate_ecc_key_pair,
    _serialize_generate_ecdsa_signature_key,
    _serialize_generate_random_bytes,
    _serialize_generate_rsa_key_pair,
    _serialize_get_public_key_from_private_key,
    _serialize_get_rsa_key_modulus_length,
    _serialize_h_mac,
    _serialize_hkdf,
    _serialize_hkdf_expand,
    _serialize_hkdf_extract,
    _serialize_kdf_counter_mode,
    _serialize_parse_public_key,
    _serialize_rsa_decrypt,
    _serialize_rsa_encrypt,
    _serialize_validate_public_key,
)


Input = TypeVar("Input")
Output = TypeVar("Output")


class AwsCryptographicPrimitives:
    """Client for AwsCryptographicPrimitives.

    :param config: Configuration for the client.
    """

    def __init__(
        self,
        config: CryptoConfig | None = None,
        dafny_client: IAwsCryptographicPrimitivesClient | None = None,
    ):
        if config is None:
            self._config = Config()
        else:
            self._config = config

        client_plugins: list[Plugin] = [
            set_config_impl,
        ]

        for plugin in client_plugins:
            plugin(self._config)

        if dafny_client is not None:
            self._config.dafnyImplInterface.impl = dafny_client

    def generate_random_bytes(
        self, input: GenerateRandomBytesInput
    ) -> bytes | bytearray:
        """Invokes the GenerateRandomBytes operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_generate_random_bytes,
            deserialize=_deserialize_generate_random_bytes,
            config=self._config,
            operation_name="GenerateRandomBytes",
        )

    def digest(self, input: DigestInput) -> bytes | bytearray:
        """Invokes the Digest operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_digest,
            deserialize=_deserialize_digest,
            config=self._config,
            operation_name="Digest",
        )

    def h_mac(self, input: HMacInput) -> bytes | bytearray:
        """Invokes the HMac operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_h_mac,
            deserialize=_deserialize_h_mac,
            config=self._config,
            operation_name="HMac",
        )

    def hkdf_extract(self, input: HkdfExtractInput) -> bytes | bytearray:
        """Invokes the HkdfExtract operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_hkdf_extract,
            deserialize=_deserialize_hkdf_extract,
            config=self._config,
            operation_name="HkdfExtract",
        )

    def hkdf_expand(self, input: HkdfExpandInput) -> bytes | bytearray:
        """Invokes the HkdfExpand operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_hkdf_expand,
            deserialize=_deserialize_hkdf_expand,
            config=self._config,
            operation_name="HkdfExpand",
        )

    def hkdf(self, input: HkdfInput) -> bytes | bytearray:
        """Invokes the Hkdf operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_hkdf,
            deserialize=_deserialize_hkdf,
            config=self._config,
            operation_name="Hkdf",
        )

    def kdf_counter_mode(self, input: KdfCtrInput) -> bytes | bytearray:
        """Invokes the KdfCounterMode operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_kdf_counter_mode,
            deserialize=_deserialize_kdf_counter_mode,
            config=self._config,
            operation_name="KdfCounterMode",
        )

    def aes_kdf_counter_mode(self, input: AesKdfCtrInput) -> bytes | bytearray:
        """Invokes the AesKdfCounterMode operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_aes_kdf_counter_mode,
            deserialize=_deserialize_aes_kdf_counter_mode,
            config=self._config,
            operation_name="AesKdfCounterMode",
        )

    def aes_encrypt(self, input: AESEncryptInput) -> AESEncryptOutput:
        """Invokes the AESEncrypt operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_aes_encrypt,
            deserialize=_deserialize_aes_encrypt,
            config=self._config,
            operation_name="AESEncrypt",
        )

    def aes_decrypt(self, input: AESDecryptInput) -> bytes | bytearray:
        """Invokes the AESDecrypt operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_aes_decrypt,
            deserialize=_deserialize_aes_decrypt,
            config=self._config,
            operation_name="AESDecrypt",
        )

    def generate_rsa_key_pair(
        self, input: GenerateRSAKeyPairInput
    ) -> GenerateRSAKeyPairOutput:
        """Invokes the GenerateRSAKeyPair operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_generate_rsa_key_pair,
            deserialize=_deserialize_generate_rsa_key_pair,
            config=self._config,
            operation_name="GenerateRSAKeyPair",
        )

    def get_rsa_key_modulus_length(
        self, input: GetRSAKeyModulusLengthInput
    ) -> GetRSAKeyModulusLengthOutput:
        """Invokes the GetRSAKeyModulusLength operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_get_rsa_key_modulus_length,
            deserialize=_deserialize_get_rsa_key_modulus_length,
            config=self._config,
            operation_name="GetRSAKeyModulusLength",
        )

    def rsa_decrypt(self, input: RSADecryptInput) -> bytes | bytearray:
        """Invokes the RSADecrypt operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_rsa_decrypt,
            deserialize=_deserialize_rsa_decrypt,
            config=self._config,
            operation_name="RSADecrypt",
        )

    def rsa_encrypt(self, input: RSAEncryptInput) -> bytes | bytearray:
        """Invokes the RSAEncrypt operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_rsa_encrypt,
            deserialize=_deserialize_rsa_encrypt,
            config=self._config,
            operation_name="RSAEncrypt",
        )

    def generate_ecdsa_signature_key(
        self, input: GenerateECDSASignatureKeyInput
    ) -> GenerateECDSASignatureKeyOutput:
        """Invokes the GenerateECDSASignatureKey operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_generate_ecdsa_signature_key,
            deserialize=_deserialize_generate_ecdsa_signature_key,
            config=self._config,
            operation_name="GenerateECDSASignatureKey",
        )

    def ecdsa_sign(self, input: ECDSASignInput) -> bytes | bytearray:
        """Invokes the ECDSASign operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_ecdsa_sign,
            deserialize=_deserialize_ecdsa_sign,
            config=self._config,
            operation_name="ECDSASign",
        )

    def ecdsa_verify(self, input: ECDSAVerifyInput) -> bool:
        """Invokes the ECDSAVerify operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_ecdsa_verify,
            deserialize=_deserialize_ecdsa_verify,
            config=self._config,
            operation_name="ECDSAVerify",
        )

    def generate_ecc_key_pair(
        self, input: GenerateECCKeyPairInput
    ) -> GenerateECCKeyPairOutput:
        """Invokes the GenerateECCKeyPair operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_generate_ecc_key_pair,
            deserialize=_deserialize_generate_ecc_key_pair,
            config=self._config,
            operation_name="GenerateECCKeyPair",
        )

    def get_public_key_from_private_key(
        self, input: GetPublicKeyFromPrivateKeyInput
    ) -> GetPublicKeyFromPrivateKeyOutput:
        """Invokes the GetPublicKeyFromPrivateKey operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_get_public_key_from_private_key,
            deserialize=_deserialize_get_public_key_from_private_key,
            config=self._config,
            operation_name="GetPublicKeyFromPrivateKey",
        )

    def validate_public_key(
        self, input: ValidatePublicKeyInput
    ) -> ValidatePublicKeyOutput:
        """Invokes the ValidatePublicKey operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_validate_public_key,
            deserialize=_deserialize_validate_public_key,
            config=self._config,
            operation_name="ValidatePublicKey",
        )

    def derive_shared_secret(
        self, input: DeriveSharedSecretInput
    ) -> DeriveSharedSecretOutput:
        """Invokes the DeriveSharedSecret operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_derive_shared_secret,
            deserialize=_deserialize_derive_shared_secret,
            config=self._config,
            operation_name="DeriveSharedSecret",
        )

    def compress_public_key(
        self, input: CompressPublicKeyInput
    ) -> CompressPublicKeyOutput:
        """Invokes the CompressPublicKey operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_compress_public_key,
            deserialize=_deserialize_compress_public_key,
            config=self._config,
            operation_name="CompressPublicKey",
        )

    def decompress_public_key(
        self, input: DecompressPublicKeyInput
    ) -> DecompressPublicKeyOutput:
        """Invokes the DecompressPublicKey operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_decompress_public_key,
            deserialize=_deserialize_decompress_public_key,
            config=self._config,
            operation_name="DecompressPublicKey",
        )

    def parse_public_key(self, input: ParsePublicKeyInput) -> ParsePublicKeyOutput:
        """Invokes the ParsePublicKey operation.

        :param input: The operation's input.
        """
        return self._execute_operation(
            input=input,
            plugins=[],
            serialize=_serialize_parse_public_key,
            deserialize=_deserialize_parse_public_key,
            config=self._config,
            operation_name="ParsePublicKey",
        )

    def _execute_operation(
        self,
        input: Input,
        plugins: list[Plugin],
        serialize: Callable[[Input, Config], DafnyRequest],
        deserialize: Callable[[DafnyResponse, Config], Output],
        config: Config,
        operation_name: str,
    ) -> Output:
        try:
            return self._handle_execution(
                input, plugins, serialize, deserialize, config, operation_name
            )
        except Exception as e:
            # Make sure every exception that we throw is an instance of ServiceError so
            # customers can reliably catch everything we throw.
            if not isinstance(e, ServiceError):
                raise ServiceError(e) from e
            raise e

    def _handle_execution(
        self,
        input: Input,
        plugins: list[Plugin],
        serialize: Callable[[Input, Config], DafnyRequest],
        deserialize: Callable[[DafnyResponse, Config], Output],
        config: Config,
        operation_name: str,
    ) -> Output:
        context: InterceptorContext[Input, None, None, None] = InterceptorContext(
            request=input,
            response=None,
            transport_request=None,
            transport_response=None,
        )
        _client_interceptors = config.interceptors
        client_interceptors = cast(
            list[Interceptor[Input, Output, DafnyRequest, DafnyResponse]],
            _client_interceptors,
        )
        interceptors = client_interceptors

        try:
            # Step 1a: Invoke read_before_execution on client-level interceptors
            for interceptor in client_interceptors:
                interceptor.read_before_execution(context)

            # Step 1b: Run operation-level plugins
            for plugin in plugins:
                plugin(config)

            _client_interceptors = config.interceptors
            interceptors = cast(
                list[Interceptor[Input, Output, DafnyRequest, DafnyResponse]],
                _client_interceptors,
            )

            # Step 1c: Invoke the read_before_execution hooks on newly added
            # interceptors.
            for interceptor in interceptors:
                if interceptor not in client_interceptors:
                    interceptor.read_before_execution(context)

            # Step 2: Invoke the modify_before_serialization hooks
            for interceptor in interceptors:
                context._request = interceptor.modify_before_serialization(context)

            # Step 3: Invoke the read_before_serialization hooks
            for interceptor in interceptors:
                interceptor.read_before_serialization(context)

            # Step 4: Serialize the request
            context_with_transport_request = cast(
                InterceptorContext[Input, None, DafnyRequest, None], context
            )
            context_with_transport_request._transport_request = serialize(
                context_with_transport_request.request, config
            )

            # Step 5: Invoke read_after_serialization
            for interceptor in interceptors:
                interceptor.read_after_serialization(context_with_transport_request)

            # Step 6: Invoke modify_before_retry_loop
            for interceptor in interceptors:
                context_with_transport_request._transport_request = (
                    interceptor.modify_before_retry_loop(context_with_transport_request)
                )

            # Step 7: Acquire the retry token.
            retry_strategy = config.retry_strategy
            retry_token = retry_strategy.acquire_initial_retry_token()

            while True:
                # Make an attempt, creating a copy of the context so we don't pass
                # around old data.
                context_with_response = self._handle_attempt(
                    deserialize,
                    interceptors,
                    context_with_transport_request.copy(),
                    config,
                    operation_name,
                )

                # We perform this type-ignored re-assignment because `context` needs
                # to point at the latest context so it can be generically handled
                # later on. This is only an issue here because we've created a copy,
                # so we're no longer simply pointing at the same object in memory
                # with different names and type hints. It is possible to address this
                # without having to fall back to the type ignore, but it would impose
                # unnecessary runtime costs.
                context = context_with_response  # type: ignore

                if isinstance(context_with_response.response, Exception):
                    # Step 7u: Reacquire retry token if the attempt failed
                    try:
                        retry_token = retry_strategy.refresh_retry_token_for_retry(
                            token_to_renew=retry_token,
                            error_info=RetryErrorInfo(
                                # TODO: Determine the error type.
                                error_type=RetryErrorType.CLIENT_ERROR,
                            ),
                        )
                    except SmithyRetryException:
                        raise context_with_response.response
                else:
                    # Step 8: Invoke record_success
                    retry_strategy.record_success(token=retry_token)
                    break
        except Exception as e:
            context._response = e

        # At this point, the context's request will have been definitively set, and
        # The response will be set either with the modeled output or an exception. The
        # transport_request and transport_response may be set or None.
        execution_context = cast(
            InterceptorContext[
                Input, Output, DafnyRequest | None, DafnyResponse | None
            ],
            context,
        )
        return self._finalize_execution(interceptors, execution_context)

    def _handle_attempt(
        self,
        deserialize: Callable[[DafnyResponse, Config], Output],
        interceptors: list[Interceptor[Input, Output, DafnyRequest, DafnyResponse]],
        context: InterceptorContext[Input, None, DafnyRequest, None],
        config: Config,
        operation_name: str,
    ) -> InterceptorContext[Input, Output, DafnyRequest, DafnyResponse | None]:
        try:
            # Step 7a: Invoke read_before_attempt
            for interceptor in interceptors:
                interceptor.read_before_attempt(context)

            # Step 7m: Involve client Dafny impl
            if config.dafnyImplInterface.impl is None:
                raise Exception("No impl found on the operation config.")

            context_with_response = cast(
                InterceptorContext[Input, None, DafnyRequest, DafnyResponse], context
            )

            context_with_response._transport_response = (
                config.dafnyImplInterface.handle_request(
                    input=context_with_response.transport_request
                )
            )

            # Step 7n: Invoke read_after_transmit
            for interceptor in interceptors:
                interceptor.read_after_transmit(context_with_response)

            # Step 7o: Invoke modify_before_deserialization
            for interceptor in interceptors:
                context_with_response._transport_response = (
                    interceptor.modify_before_deserialization(context_with_response)
                )

            # Step 7p: Invoke read_before_deserialization
            for interceptor in interceptors:
                interceptor.read_before_deserialization(context_with_response)

            # Step 7q: deserialize
            context_with_output = cast(
                InterceptorContext[Input, Output, DafnyRequest, DafnyResponse],
                context_with_response,
            )
            context_with_output._response = deserialize(
                context_with_output._transport_response, config
            )

            # Step 7r: Invoke read_after_deserialization
            for interceptor in interceptors:
                interceptor.read_after_deserialization(context_with_output)
        except Exception as e:
            context._response = e

        # At this point, the context's request and transport_request have definitively been set,
        # the response is either set or an exception, and the transport_resposne is either set or
        # None. This will also be true after _finalize_attempt because there is no opportunity
        # there to set the transport_response.
        attempt_context = cast(
            InterceptorContext[Input, Output, DafnyRequest, DafnyResponse | None],
            context,
        )
        return self._finalize_attempt(interceptors, attempt_context)

    def _finalize_attempt(
        self,
        interceptors: list[Interceptor[Input, Output, DafnyRequest, DafnyResponse]],
        context: InterceptorContext[Input, Output, DafnyRequest, DafnyResponse | None],
    ) -> InterceptorContext[Input, Output, DafnyRequest, DafnyResponse | None]:
        # Step 7s: Invoke modify_before_attempt_completion
        try:
            for interceptor in interceptors:
                context._response = interceptor.modify_before_attempt_completion(
                    context
                )
        except Exception as e:
            context._response = e

        # Step 7t: Invoke read_after_attempt
        for interceptor in interceptors:
            try:
                interceptor.read_after_attempt(context)
            except Exception as e:
                context._response = e

        return context

    def _finalize_execution(
        self,
        interceptors: list[Interceptor[Input, Output, DafnyRequest, DafnyResponse]],
        context: InterceptorContext[
            Input, Output, DafnyRequest | None, DafnyResponse | None
        ],
    ) -> Output:
        try:
            # Step 9: Invoke modify_before_completion
            for interceptor in interceptors:
                context._response = interceptor.modify_before_completion(context)

        except Exception as e:
            context._response = e

        # Step 11: Invoke read_after_execution
        for interceptor in interceptors:
            try:
                interceptor.read_after_execution(context)
            except Exception as e:
                context._response = e

        # Step 12: Return / throw
        if isinstance(context.response, Exception):
            raise context.response

        # We may want to add some aspects of this context to the output types so we can
        # return it to the end-users.
        return context.response
